#include "mainwindow.h"
#include "ui_mainwindow.h"
#include <QThread>
#include "zkglobaldef.h"
#include <QTime>
#include <QEventLoop>

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    
    LOGDEBUG;
    pOperate  =  new Operate;
    pZKBiometric = new ZKBiometric;
    pZKSensor = new ZKSensor;

    pZKBiometric->setEditId(ui->lineEdit_id);//control UI lineEditId point to class ZKBiometric;
    pOperate->setObjects(pZKBiometric,pZKSensor);
    LOGDEBUG;

    QObject::connect( ui->pushButton_open,            SIGNAL(clicked()),pOperate,SLOT(slotButtonOpen()));
    QObject::connect( ui->pushButton_close,            SIGNAL(clicked()),pOperate,SLOT(slotButtonClose()));
    QObject::connect( ui->pushButton_enroll,           SIGNAL(clicked()),pOperate,SLOT(slotButtonEnroll()));
    QObject::connect( ui->pushButton_delete,           SIGNAL(clicked()),pOperate,SLOT(slotButtonDelete()));
    QObject::connect( ui->pushButton_clear,             SIGNAL(clicked()),pOperate,SLOT(slotButtonClear()));
    QObject::connect( ui->pushButton_verify,            SIGNAL(clicked()),pOperate,SLOT(slotButtonVerify()));
    QObject::connect( ui->pushButton_identify,         SIGNAL(clicked()),pOperate,SLOT(slotButtonIdentify()));
    QObject::connect( ui->pushButton_getParam,      SIGNAL(clicked()),pOperate,SLOT(slotButtonGetParam()));

    QObject::connect( pZKSensor, SIGNAL(signalStartLoopCapture()),pZKSensor,SLOT(slotStartLoopCapture()));
    QObject::connect( pZKSensor, SIGNAL(signalSendMsg(QString)),this,SLOT(slotRecvMsg(QString)));
    QObject::connect( pZKSensor, SIGNAL(signalSendImage(QImage)),this,SLOT(slotRecvImage(QImage)),Qt::BlockingQueuedConnection);

    QObject::connect( pZKSensor, SIGNAL(signalSendImageEx(unsigned char*,int,int)),pZKBiometric,SLOT(slotRecvImage(unsigned char*,int,int)),Qt::BlockingQueuedConnection);

    QObject::connect( pZKBiometric, SIGNAL(signalSendExtractSuccess()),pOperate,SLOT(slotRecvExtractSuccess()),Qt::UniqueConnection);
    QObject::connect( pZKBiometric, SIGNAL(signalSendMsg(QString)),this,SLOT(slotRecvMsg(QString)));
    QObject::connect( pZKBiometric, SIGNAL(signalSendRect(int*,int,int)),this,SLOT(slotRecvRect(int*,int,int)),Qt::BlockingQueuedConnection);

    LOGDEBUG;

    threadZKSensor  = new QThread;
    pZKSensor->moveToThread(threadZKSensor);
    threadZKSensor->start();

    threadZKBiometric  =new QThread;
    pZKBiometric->moveToThread(threadZKBiometric);
    threadZKBiometric->start();

    threadOperate  =new QThread;
    pOperate->moveToThread(threadOperate);
    threadOperate->start();
    LOGDEBUG;
}

MainWindow::~MainWindow()
{
    LOGDEBUG;
    delete ui;
    LOGDEBUG;



    threadOperate->quit();    LOGDEBUG;
    threadOperate->wait();    LOGDEBUG;
    delete threadOperate;    LOGDEBUG;

    threadZKSensor->quit();    LOGDEBUG;
    threadZKSensor->wait();    LOGDEBUG;
    delete threadZKSensor;    LOGDEBUG;

    threadZKBiometric->quit();    LOGDEBUG;
    threadZKBiometric->wait();    LOGDEBUG;
    delete threadZKBiometric;    LOGDEBUG;

    delete pZKBiometric;
    pZKBiometric = NULL;
    delete pZKSensor;
    pZKSensor = NULL;
    delete pOperate;
    pOperate = NULL;


}
//void MainWindow:: closeEvent(QCloseEvent *event)
//{
//       QEventLoop loop;

//       QTimer::singleShot(1*1000,pZKBiometric,SLOT(terminate()));
//       QTimer::singleShot(1*1000,pZKSensor,SLOT(terminate()));
//       QTimer::singleShot(1*1000,pOperate,SLOT(terminate()));
//       QTimer::singleShot(1.5*1000,&loop,SLOT(quit()));
//       QTimer::singleShot(2*1000,this,SLOT(close()));
//       loop.exec();
//       event->accept();
//}



void MainWindow::slotRecvMsg(QString msg)
{
    LOGDEBUG;
    QTextCursor cursor = ui->textEdit->textCursor();
    cursor.movePosition(QTextCursor::End);
    ui->textEdit->setTextCursor(cursor);
    ui->textEdit ->append(msg);
    LOGDEBUG;
}
void MainWindow::slotRecvImage(QImage image)
{
    LOGDEBUG;
    ui->label_palm_image ->setPixmap(QPixmap::fromImage(image).scaled( ui->label_palm_image->width(),ui->label_palm_image->height()));
    LOGDEBUG;
}
void MainWindow::slotRecvRect(int* rect,int width,int height)
{
    if(NULL != rect)
    {
           //X 轴缩放比率
           float xZoom = float(ui->label_palm_image->width())/float(width);
           //Y 轴缩放比率
           float yZoom = float(ui->label_palm_image->height())/float(height);
           //根据控件的实际尺寸大小，等比例显示手掌矩形框(等比例缩放手掌矩形框)
           for(int i = 0;i<4;i++)
           {
               rect[i*2] = rect[i*2]*xZoom;
               rect[i*2+1] = rect[i*2+1]*yZoom;
           }
           ui->label_palm_image->setPalmRect(rect);
           ui->label_palm_image->update();
    }
}
