#include "zkbiometric.h"
#include <QLibrary>
#include <QDateTime>
#include "../../zkglobaldef.h"
#include "include/ZKPalmErrno.h"

#ifdef _WIN32
#define ZKPalmAPI_LIBNAME "ZKIrpalmServer.dll"
#endif
#ifdef __linux
#define ZKPalmAPI_LIBNAME "libzkirpalmserver.so"
#endif

#define TAG "zkbiometric: "


int k=0;

ZKBiometric::ZKBiometric(QObject *parent) : QObject(parent)
{
    m_hDbcache = NULL;
    m_nImgWidth = 480;
    m_nImgHeight = 640;
    m_pcImgRaw = new unsigned char[MAX_IMG_SIZE];
    m_pcAddTemplate = new unsigned char[MAX_ADD_TEMPLATE_SIZE];
    m_pcLastAddTemplate = new unsigned char[MAX_ADD_TEMPLATE_SIZE];
    m_pcVerTemplate = new unsigned char[MAX_VER_TEMPLATE_SIZE];
    m_gpcAddTemplates = new unsigned char*[MAX_MERGE_TEMPLATE_COUNT];
    for(int i=0;i<MAX_MERGE_TEMPLATE_COUNT;i++)
    {
        m_gpcAddTemplates[i] = new unsigned char[MAX_ADD_TEMPLATE_SIZE];
    }
    m_gnImgRect = new int[8];
}
ZKBiometric::~ZKBiometric()
{
    m_hDbcache = NULL;
    m_nImgWidth = 0;
    m_nImgHeight = 0;
    if(m_pcImgRaw)
    {
        delete m_pcImgRaw;
        m_pcImgRaw = NULL;
    }
    if(m_pcAddTemplate)
    {
        delete m_pcAddTemplate;
        m_pcAddTemplate = NULL;
    }
    if(m_pcLastAddTemplate)
    {
        delete m_pcLastAddTemplate;
        m_pcLastAddTemplate = NULL;
    }
    if(m_pcVerTemplate)
    {
        delete m_pcVerTemplate;
        m_pcVerTemplate = NULL;
    }

    for(int i=0;i<MAX_MERGE_TEMPLATE_COUNT;i++)
    {
        if(NULL != m_gpcAddTemplates[i])
        {
            delete m_gpcAddTemplates[i];
            m_gpcAddTemplates[i] = NULL;
        }
    }
    if(m_gpcAddTemplates)
    {
        delete m_gpcAddTemplates;
        m_gpcAddTemplates = NULL;
    }
    if(m_gnImgRect)
    {
        delete m_gnImgRect;
        m_gnImgRect = NULL;
    }

}
void ZKBiometric::setEditId(QLineEdit* lineEdit)
{
    this->lineEdit = lineEdit;
}

bool ZKBiometric::loadLibrary()
{

    QString msg;
    QLibrary libzkbiometric(ZKPalmAPI_LIBNAME);
    if(!libzkbiometric.load())
    {
        msg=QString::fromLocal8Bit("biometric library %1 load failed\n").arg(ZKPalmAPI_LIBNAME);
        signalSendMsg(msg);
        LOGDEBUG << msg;
        return false;
    }
    ZKPalmServiceVersion = (P_ZKPalmServiceVersion)libzkbiometric.resolve("ZKPalmServiceVersion");
    ZKPalmServiceInit = (P_ZKPalmServiceInit)libzkbiometric.resolve("ZKPalmServiceInit");
    ZKPalmServiceFinal = (P_ZKPalmServiceFinal)libzkbiometric.resolve("ZKPalmServiceFinal");
    ZKPalmServiceSetParam = (P_ZKPalmServiceSetParam)libzkbiometric.resolve("ZKPalmServiceSetParam");
    ZKPalmServiceExtractTemplate = (P_ZKPalmServiceExtractTemplate)libzkbiometric.resolve("ZKPalmServiceExtractTemplate");
    ZKPalmServiceVerify = (P_ZKPalmServiceVerify)libzkbiometric.resolve("ZKPalmServiceVerify");
    ZKPalmServiceVerifyByID = (P_ZKPalmServiceVerifyByID)libzkbiometric.resolve("ZKPalmServiceVerifyByID");
    ZKPalmServiceMergeTemplates = (P_ZKPalmServiceMergeTemplates)libzkbiometric.resolve("ZKPalmServiceMergeTemplates");
    ZKPalmServiceDBAdd = (P_ZKPalmServiceDBAdd)libzkbiometric.resolve("ZKPalmServiceDBAdd");
    ZKPalmServiceDBDel = (P_ZKPalmServiceDBDel)libzkbiometric.resolve("ZKPalmServiceDBDel");
    ZKPalmServiceDBCount = (P_ZKPalmServiceDBCount)libzkbiometric.resolve("ZKPalmServiceDBCount");
    ZKPalmServiceDBClear = (P_ZKPalmServiceDBClear)libzkbiometric.resolve("ZKPalmServiceDBClear");
    ZKPalmServiceDBIdentify = (P_ZKPalmServiceDBIdentify)libzkbiometric.resolve("ZKPalmServiceDBIdentify");
    ZKPalmServiceEnrollDBIdentify = (P_ZKPalmServiceEnrollDBIdentify)libzkbiometric.resolve("ZKPalmServiceEnrollDBIdentify");
    if( NULL == ZKPalmServiceVersion ||
            NULL ==   ZKPalmServiceInit ||
            NULL == ZKPalmServiceFinal ||
            NULL == ZKPalmServiceSetParam ||
            NULL == ZKPalmServiceExtractTemplate ||
            NULL == ZKPalmServiceVerify ||
            NULL == ZKPalmServiceVerifyByID ||
            NULL == ZKPalmServiceMergeTemplates ||
            NULL == ZKPalmServiceDBAdd ||
            NULL == ZKPalmServiceDBDel ||
            NULL == ZKPalmServiceDBCount ||
            NULL == ZKPalmServiceDBClear ||
            NULL == ZKPalmServiceDBIdentify ||
            NULL == ZKPalmServiceEnrollDBIdentify)
    {
        msg=QString::fromLocal8Bit("biometric library %1 func relolve failed\n").arg(ZKPalmAPI_LIBNAME);
        signalSendMsg(msg);
        LOGDEBUG << msg;
        return false;
    }
    return true;
}
void ZKBiometric::init()
{
    int nRet = 0;
    QString msg = NULL;
    int nVersionDate = 0;//版本号的日期
    int nMajor = 0;//主版本号
    int nMinor = 0;//次版本号
    char szVersion[64] = {0x0};
    bool bIsMallocDb = true;

    if(!loadLibrary())
    {
        return  ;
    }

    if(NULL != m_hDbcache)
    {
        msg = QString::fromLocal8Bit("算法已经初始化\n");
    }
    else
    {
        nRet =  ZKPalmServiceVersion(&nVersionDate, &nMajor, &nMinor,szVersion, 64);
        if(ZKPALM_ERR_OK != nRet)
        {
            msg = QString::fromLocal8Bit("获取算法版本信息失败\n");
        }
        else
        {
            msg = QString::fromLocal8Bit("算法版日期:%1 版本号:%2.%3").arg(nVersionDate).arg(nMajor).arg(nMinor);
            LOGDEBUG << msg;
            signalSendMsg(msg);

            nRet = ZKPalmServiceInit(&m_hDbcache,m_nImgWidth,m_nImgHeight,bIsMallocDb);
            LOGDEBUG ;
            if(ZKPALM_ERR_OK != nRet)
            {
                msg = QString::fromLocal8Bit("算法初始化失败,error code:%1\n").arg(nRet);
            }
            else
            {
                msg = QString::fromLocal8Bit("算法初始化成功\n");
            }
        }
    }
    LOGDEBUG << msg;
    signalSendMsg(msg);
    return;
}
void ZKBiometric::free()
{
    int nRet = 0;
    QString msg = NULL;
    if(NULL == m_hDbcache)
    {
        msg = QString::fromLocal8Bit("算法未初始化,请先初始化算法");
    }
    else
    {
        mutex.lock();
        nRet  =  ZKPalmServiceFinal();
        m_hDbcache = NULL;
        mutex.unlock();
        if(ZKPALM_ERR_OK != nRet)
        {
            msg = QString::fromLocal8Bit("算法释放失败,error code:%1").arg(nRet);
        }
        else
        {
            msg = QString::fromLocal8Bit("算法释放成功");
        }
    }
    LOGDEBUG << msg;
    signalSendMsg(msg);
}
void ZKBiometric::add()
{
    int nRet  = -1;
    QString msg = NULL;
    int nAddCnt = 0;
    int extractType  = 1;
    long tickStart = 0;
    long tickCurrent = 0;
    int nRawTemplateSize = 0;
    int nVerTemplateSize = 0;
    int nAddTemplateSize = 0;
    int nQuality = 0;
    char szId[64] = {0x0};

    if(NULL == m_hDbcache)
    {
        msg = QString::fromLocal8Bit("算法未初始化,请先初始化算法\n");
    }
    else
    {
        if(this->lineEdit->text().isEmpty())
        {
            msg = QString::fromLocal8Bit("手掌 ID 为空，请输入你需要登记的ID");
            LOGDEBUG;
            signalSendMsg(msg);
            return;
        }
        else
        {
            tickStart = QDateTime::currentMSecsSinceEpoch();
            for(;;)
            {
                tickCurrent = QDateTime::currentMSecsSinceEpoch();
                if(tickCurrent -tickStart>1000*10) //10 seccond do add
                {
                    msg = QString::fromLocal8Bit("10秒超时登记失败\n");
                    LOGDEBUG;
                    signalSendMsg(msg);
                    break;
                }

                nRawTemplateSize = MAX_ADD_TEMPLATE_SIZE;
                nVerTemplateSize = MAX_VER_TEMPLATE_SIZE;
                nQuality = 0;
                memset(m_gpcAddTemplates[nAddCnt],0x0,MAX_ADD_TEMPLATE_SIZE);
                memset(m_pcVerTemplate,0x0,MAX_VER_TEMPLATE_SIZE);
                mutex.lock();
                nRet = ZKPalmServiceExtractTemplate(0, m_pcImgRaw,  extractType, m_gpcAddTemplates[nAddCnt],&nRawTemplateSize, m_pcVerTemplate, &nVerTemplateSize, &nQuality,m_gnImgRect);
                mutex.unlock();
                if(ZKPALM_ERR_OK != nRet)
                {
                    msg = QString::fromLocal8Bit("提取模板失败,error code:%1").arg(nRet);
                    LOGDEBUG <<msg;
                    //signalSendMsg(msg);
                    zkusleep(100*1000);
                    continue ;
                }
                else
                {
                    zkusleep(500*1000);
                    tickStart = QDateTime::currentMSecsSinceEpoch();
                    msg = QString::fromLocal8Bit("提取第%1个模板成功").arg(nAddCnt+1);
                    LOGDEBUG <<msg;
                    signalSendMsg(msg);

                    nAddCnt++;
                    if(MAX_MERGE_TEMPLATE_COUNT == nAddCnt)
                    {
                        nAddTemplateSize = MAX_ADD_TEMPLATE_SIZE;
                        memset(m_pcAddTemplate,0x0,MAX_ADD_TEMPLATE_SIZE);
                        mutex.lock();
                        nRet  = ZKPalmServiceMergeTemplates(0,m_gpcAddTemplates,MAX_MERGE_TEMPLATE_COUNT,m_pcAddTemplate,&nAddTemplateSize);
                        mutex.unlock();
                        if(ZKPALM_ERR_OK != nRet)
                        {
                            msg = QString::fromLocal8Bit("模板合成失败,error code:%1\n").arg(nRet);
                        }
                        else
                        {
                            msg = QString::fromLocal8Bit("模板合成成功");
                            LOGDEBUG <<msg;
                            signalSendMsg(msg);

                            sprintf(szId,"%d",this->lineEdit->text().toInt());
                            mutex.lock();
                            nRet = ZKPalmServiceDBAdd(0, (const char*)szId, m_pcAddTemplate,1);
                            mutex.unlock();
                            if(0 != nRet)
                            {
                                msg = QString::fromLocal8Bit("ID:%1登记模板失败,error code:%2\n").arg(szId).arg(nRet);
                            }
                            else
                            {
                                msg = QString::fromLocal8Bit("ID:%1 登记模板成功!\n").arg(szId);
                                memcpy(m_pcLastAddTemplate,m_pcAddTemplate,MAX_ADD_TEMPLATE_SIZE);
                            }
                        }
                        break ;
                    }
                }
            }
        }
    }
    LOGDEBUG <<msg;
    signalSendMsg(msg);
    return;
}
void ZKBiometric::del()
{
    int nRet = 0;
    QString msg = NULL;
    char szId[64] = {0x0};
    if(NULL == m_hDbcache)
    {
        msg = QString::fromLocal8Bit("算法未初始化,请先初始化算法\n");
    }
    else
    {
        if(this->lineEdit->text().isEmpty())
        {
            msg = QString::fromLocal8Bit("手掌 ID 为空，请输入你需要删除的ID");
        }
        else
        {
            sprintf(szId,"%d",this->lineEdit->text().toInt());
            mutex.lock();
            nRet =  ZKPalmServiceDBDel(0, szId);
            mutex.unlock();
            if(ZKPALM_ERR_OK != nRet)
            {
                msg = QString::fromLocal8Bit("ID:%1删除失败,error code:%2\n").arg(szId).arg(nRet);
            }
            else
            {
                msg = QString::fromLocal8Bit("ID:%1删除成功\n").arg(szId);
            }
        }
    }
    LOGDEBUG << msg;
    signalSendMsg(msg);
}

void ZKBiometric::clear()
{
    int nRet = 0;
    QString msg = NULL;
    if(NULL == m_hDbcache)
    {
        msg = QString::fromLocal8Bit("算法未初始化,请先初始化算法\n");
    }
    else
    {
        mutex.lock();
        nRet  =  ZKPalmServiceDBClear(0);
        mutex.unlock();
        if(ZKPALM_ERR_OK != nRet)
        {
            msg = QString::fromLocal8Bit("数据清空失败,error code:%1\n").arg(nRet);
        }
        else
        {
            msg = QString::fromLocal8Bit("数据清空成功\n");
        }
    }
    LOGDEBUG << msg;
    signalSendMsg(msg);
}
void ZKBiometric::verify()
{
    int nRet  = -1;
    QString msg = NULL;
    int extractType  = 2;
    long tickStart = 0;
    long tickCurrent = 0;
    int nVerTemplateSize = 0;
    int nAddTemplateSize = 0;
    int nQuality = 0;
    char szId[64] = {0x0};
    bool bIsLineEditEmpty = false;

    if(NULL == m_hDbcache)
    {
        msg = QString::fromLocal8Bit("算法未初始化,请先初始化算法\n");
    }
    else
    {
        tickStart = QDateTime::currentMSecsSinceEpoch();
        for(;;)
        {
            tickCurrent = QDateTime::currentMSecsSinceEpoch();
            if(tickCurrent -tickStart>1000*10) //10 seccond do verify
            {
                msg = QString::fromLocal8Bit("10秒超时1:1比对失败");
                break;
            }

            nQuality = 0;
            nAddTemplateSize = MAX_ADD_TEMPLATE_SIZE;
            nVerTemplateSize = MAX_VER_TEMPLATE_SIZE;
            memset(m_pcAddTemplate,0x0,MAX_ADD_TEMPLATE_SIZE);
            memset(m_pcVerTemplate,0x0,MAX_VER_TEMPLATE_SIZE);
            mutex.lock();
            nRet = ZKPalmServiceExtractTemplate(0, m_pcImgRaw,  extractType, m_pcAddTemplate,&nAddTemplateSize, m_pcVerTemplate, &nVerTemplateSize, &nQuality,m_gnImgRect);
            mutex.unlock();

            if(ZKPALM_ERR_OK != nRet)
            {
                msg = QString::fromLocal8Bit("提取模板失败,error code:%1").arg(nRet);
                LOGDEBUG <<msg;
                //signalSendMsg(msg);
                zkusleep(100*1000);
                continue ;
            }
            else
            {
                msg = QString::fromLocal8Bit("提取模板成功").arg(nRet);
                LOGDEBUG <<msg;
                signalSendMsg(msg);

                bIsLineEditEmpty = this->lineEdit->text().isEmpty();
                if( bIsLineEditEmpty )
                {
                    mutex.lock();
                    nRet = ZKPalmServiceVerify(0,m_pcLastAddTemplate,m_pcVerTemplate);
                    mutex.unlock();                }
                else
                {
                    sprintf(szId,"%d",this->lineEdit->text().toInt());
                    mutex.lock();
                    nRet = ZKPalmServiceVerifyByID(0,m_pcVerTemplate,(const char*)szId);
                    mutex.unlock();

                }
                if(ZKPALM_ERR_OK > nRet)
                {
                    if(bIsLineEditEmpty)
                    {
                        msg = QString::fromLocal8Bit("1:1比对最后一个登记模板失败,error code:%1\n").arg(nRet);
                    }
                    else
                    {
                        msg = QString::fromLocal8Bit("1:1比对ID:%1失败,error code:%2\n").arg(szId).arg(nRet);
                    }
                }
                else
                {
                    if(SUCCESS_SCORE>nRet)
                    {
                        if(bIsLineEditEmpty)
                        {
                            msg = QString::fromLocal8Bit("1:1比对最后一个登记模板失败,score:%1\n").arg(nRet);
                        }
                        else
                        {
                            msg = QString::fromLocal8Bit("1:1比对ID:%1失败,score:%2\n").arg(szId).arg(nRet);
                        }
                    }
                    else
                    {
                        if(bIsLineEditEmpty)
                        {
                            msg = QString::fromLocal8Bit("1:1比对最后一个登记模板成功,score:%1\n").arg(nRet);
                        }
                        else
                        {
                            msg = QString::fromLocal8Bit("1:1比对ID:%1成功,score:%2\n").arg(szId).arg(nRet);
                        }
                    }
                }
                break ;
            }
        }
    }
    LOGDEBUG <<msg;
    signalSendMsg(msg);
    return;
}
void ZKBiometric::identify()
{
    int nRet  = -1;
    QString msg = NULL;
    int extractType  = 2;
    long tickStart = 0;
    long tickCurrent = 0;
    int nVerTemplateSize = 0;
    int nAddTemplateSize = 0;
    int nQuality = 0;
    char szId[64] = {0x0};

    if(NULL == m_hDbcache)
    {
        msg = QString::fromLocal8Bit("算法未初始化,请先初始化算法\n");
    }
    else
    {
        tickStart = QDateTime::currentMSecsSinceEpoch();
        for(;;)
        {
            tickCurrent = QDateTime::currentMSecsSinceEpoch();
            if(tickCurrent -tickStart>1000*10) //10 seccond do identify
            {
                msg = QString::fromLocal8Bit("10秒超时1:N比对失败");
                break;
            }
            nQuality = 0;
            nAddTemplateSize = MAX_ADD_TEMPLATE_SIZE;
            nVerTemplateSize = MAX_VER_TEMPLATE_SIZE;
            memset(m_pcAddTemplate,0x0,MAX_ADD_TEMPLATE_SIZE);
            memset(m_pcVerTemplate,0x0,MAX_VER_TEMPLATE_SIZE);
            mutex.lock();
            nRet = ZKPalmServiceExtractTemplate(0, m_pcImgRaw,  extractType, m_pcAddTemplate,&nAddTemplateSize, m_pcVerTemplate, &nVerTemplateSize, &nQuality,m_gnImgRect);
            mutex.unlock();

            if(ZKPALM_ERR_OK != nRet)
            {
                msg = QString::fromLocal8Bit("提取模板失败,error code:%1").arg(nRet);
                LOGDEBUG <<msg;
                //signalSendMsg(msg);
                zkusleep(100*1000);
                continue ;
            }
            else
            {
                msg = QString::fromLocal8Bit("提取模板成功");
                LOGDEBUG <<msg;
                signalSendMsg(msg);

                if(1 == extractType) //use verify template to identify
                {
                    mutex.lock();
                    nRet = ZKPalmServiceEnrollDBIdentify (0, m_pcAddTemplate, szId, SUCCESS_SCORE, 1000);
                    mutex.unlock();
                }
                else if(2 == extractType) //use identify template to identify
                {
                    mutex.lock();
                    nRet = ZKPalmServiceDBIdentify(0, m_pcVerTemplate, szId, SUCCESS_SCORE, 1000);
                    mutex.unlock();
                }
                if(ZKPALM_ERR_OK > nRet)
                {
                    msg = QString::fromLocal8Bit("1:N比对失败,error code:%1\n").arg(nRet);
                }
                else
                {
                    if(SUCCESS_SCORE>nRet)
                    {
                        msg = QString::fromLocal8Bit("1:N比对失败,score:%1\n").arg(nRet);
                    }
                    else
                    {
                        msg = QString::fromLocal8Bit("1:N比对成功,id:%1,score:%2\n").arg(szId).arg(nRet);
                    }
                }
                break ;
            }
        }
    }
    LOGDEBUG <<msg;
    signalSendMsg(msg);
    return;
}

void ZKBiometric::getRect(unsigned char*image,int width,int height)
{
    int nRet = 0;
    int nExtractType  = 1;
    int nQuality = 0;
    int nImgRect[8] = {0x0};
    int nAddTemplateSize = MAX_ADD_TEMPLATE_SIZE;
    int nVerTemplateSize = MAX_VER_TEMPLATE_SIZE;

    if(NULL == m_hDbcache)
    {
    }
    else
    {
        unsigned char* pcAddTemplate = new unsigned char[MAX_ADD_TEMPLATE_SIZE];
        unsigned char* pcVerTemplate = new unsigned char[MAX_VER_TEMPLATE_SIZE];
        memset(pcAddTemplate,0x0,MAX_ADD_TEMPLATE_SIZE);
        memset(pcVerTemplate,0x0,MAX_VER_TEMPLATE_SIZE);
        nRet = ZKPalmServiceExtractTemplate(0, image,  nExtractType, pcAddTemplate,&nAddTemplateSize, pcVerTemplate, &nVerTemplateSize, &nQuality,nImgRect);
        if(NULL != pcAddTemplate)
        {
            delete[] pcAddTemplate;
            pcAddTemplate = NULL;
        }
        if(NULL !=pcVerTemplate)
        {
            delete[] pcVerTemplate;
            pcVerTemplate = NULL;
        }
        if(ZKPALM_ERR_OK == nRet)
        {
            signalSendExtractSuccess();
            signalSendRect(nImgRect,width,height);
        }
    }
}

void ZKBiometric::slotRecvImage( unsigned char* image,int width,int height)
{
    LOGDEBUG;
    if(!mutex.tryLock())
    {
        return;
    }
    getRect(image,width,height);
    m_nImgWidth = width;
    m_nImgHeight = height;
    memcpy(m_pcImgRaw,image,width*height);
    mutex.unlock();
}

