﻿#ifndef ZKPALMCAP_H
#define ZKPALMCAP_H


/**
*	@file		ZKPalmCap.h
*	@brief		手掌采集库
*	@author		yiming
*	@date		2020-06-12
*	@version	3.0
*	@par	版权：
*				ZKTeco
*	@par	历史版本
				
*
*	@note		
*
*/

#if defined(_WIN32) || defined(__WIN32__)
#if defined(palm_EXPORTS)
#define  ZKINTERFACE
#else
#define  ZKINTERFACE 
#endif
#elif defined(linux) || defined(__linux)
#if __GNUC__ >= 4
#define ZKINTERFACE __attribute__ ((visibility ("default")))
#else
#define ZKINTERFACE
#endif
#endif


#ifndef APICALL
#ifdef _WIN32
#define APICALL __stdcall
#else
#define APICALL
#endif
#endif


	/*! 设置外接状态灯板PAR200(只写)
	*
	*	类型说明如下：
	*	value			|	type		|	description of value
	*	----------------|---------------|-------------------------------
	*	0				|	int			|	关闭所有状态灯
	*	1				|	int			|	打开红色状态灯
	*	2				|	int			|	打开绿色状态灯
	*	3				|	int			|	打开蓝色状态灯
	* @note 
	* 连接成功后设置
	*/
#define PARAM_CODE_BACKLIGHT_SWITCH	       2004 //控制LED状态灯
#define PARAM_CODE_GETSN	                1103  //获取SN号
#define PARAM_DETECTED			            2000 //探测手掌
#define PARAM_SENSOR_MODE		            2003 //采集器模式
#define PARAM_CODE_CLOSENIRLED	            2005 //关闭进红外Led灯	
#define PARAM_CODE_GETSLEEPSTATUS	        2008 //获取PAR200 的休眠状态值
#define PARAM_CODE_SETPALMEXTRACTSTATUS   	2010 //设置手掌算法提取手掌模板是否成功(通知设备手掌是否离开)
#define PARAM_CODE_FWVERSION                10010 // 获取固件版本号（只读）
#define PARAM_CODE_SETPALMCAPMODE   	    2011 //设置手掌的采集的模式
#define PARAM_CODE_CLOSELEDTIMEOUT			2012 // 设置探测模式下关闭背光灯的超时时间

#if 0
#ifdef __cplusplus
extern "C"
{
#endif 

	/**
	*	@brief	初始化采集库
	*	@param	:
	*		无
	*	@return
	*	返回值说明如下：
	*	value			|	type		|	description of value
	*	----------------|---------------|-------------------------------
	*	0				|	int			|	成功
	*	其他			|	int			|	失败
	*	@note 
	*		调用所有函数(除palmSensorGetVersion)前，请先初始化设备列表
	*/
	ZKINTERFACE int APICALL palmSensorInit();


	/**
	*	@brief	释放采集库
	*	@param	:
	*		无
	*	@return
	*	返回值说明如下：
	*	value			|	type		|	description of value
	*	----------------|---------------|-------------------------------
	*	0				|	int			|	0 成功
	*	其他			|	int			|	失败
	*	@note 
	*/
	ZKINTERFACE int APICALL palmSensorFree();

	/**
	*	@brief	获取采集器数量
	*	@param	:
	*		无
	*	@return
	*	返回值说明如下：
	*	value			|	type		|	description of value
	*	----------------|---------------|-------------------------------
	*	0				|	int			|	当前无采集器接入
	*	>0				|	int			|	当前接入采集器个数
	*	@note 
	*/
	ZKINTERFACE int APICALL palmSensorGetCount();

	/**
	*	@brief	获取采集库SDK版本号
	*	@param	:
	*	参数说明如下表
	*	name			|	type		|	param direction		|	description of param
	*	----------------|---------------|-----------------------|------------------------
	*	version			|	char*		|	[out]				|	缓冲区指针(>=20Bytes)
	*	len				|	int			|	[in]				|	外部分配缓冲区长度
	*	@return
	*		无
	*	@note 
	*/
	ZKINTERFACE int APICALL palmSensorGetVersion(char *version, int len);
	

	/**
	*	@brief	打开采集器
	*	@param	:
	*	参数说明如下表
	*	name			|	type		|	param direction		|	description of param
	*	----------------|---------------|-----------------------|------------------------
	*	index			|	int			|	[in]				|	设备索引0~(n-1),n=palmSensorGetCount
	*	@return
	*	返回值说明如下：
	*	value			|	type		|	description of value
	*	----------------|---------------|-------------------------------
	*	NULL			|	void*		|	打开失败
	*	其他			|	void*		|	成功
	*	@note 
	*/
	ZKINTERFACE void* APICALL palmSensorOpen(int index);


	/**
	*	@brief	关闭采集器
	*	@param	:
	*	参数说明如下表
	*	name			|	type		|	param direction		|	description of param
	*	----------------|---------------|-----------------------|------------------------
	*	handle			|	void*		|	[in]				|	设备操作实例句柄
	*	@return
	*	返回值说明如下：
	*	value			|	type		|	description of value
	*	----------------|---------------|-------------------------------
	*	0				|	int			|	成功
	*	其他			|	int			|	失败
	*	@note 
	*/
	ZKINTERFACE int APICALL palmSensorClose(void *handle);

	/**
	*	@brief	采集指纹图像
	*	@param	:
	*	参数说明如下表
	*	name			|	type		|	param direction		|	description of param
	*	----------------|---------------|-----------------------|------------------------
	*	handle			|	void*		|	[in]				|	设备操作实例句柄
	*	imageBuffer		|unsigned char* |	[out]				|	缓冲区指针（>=(width*height*3) Bytes)
	*	imageBufferSize	|	int			|	[in]				|	分配缓冲区大小
	*	@return
	*	返回值说明如下：
	*	value			|	type		|	description of value
	*	----------------|---------------|-------------------------------
	*	>0				|	int			|	成功(实际图像数据大小)
	*	其他			|	int			|	失败
	*	@note 
	*/
	ZKINTERFACE int APICALL palmSensorCapture(void *handle, unsigned char *imageBuffer, int imageBufferSize);

	/**
	*	@brief	获取设备参数
	*	@param	:
	*	参数说明如下表
	*	name			|	type		|	param direction		|	description of param
	*	----------------|---------------|-----------------------|------------------------
	*	handle			|	void*		|	[in]				|	设备操作实例句柄
	*	paramCode		|	int			|	[out]				|	参数类型
	*	@return
	*	返回值说明如下：
	*	value			|	type		|	description of value
	*	----------------|---------------|-------------------------------
	*	>=0				|	int			|	成功（参数值）
	*	其他			|	int			|	失败
	*	@note 
	*/
	ZKINTERFACE int APICALL palmSensorGetParameter(void *handle, int paramCode);

	/**
	*	@brief	设置设备参数
	*	@param	:
	*	参数说明如下表
	*	name			|	type		|	param direction		|	description of param
	*	----------------|---------------|-----------------------|------------------------
	*	handle			|	void*		|	[in]				|	设备操作实例句柄
	*	paramCode		|	int			|	[in]				|	参数类型
	*	paramValue		|	int			|	[in]				|	参数值
	*	@return
	*	返回值说明如下：
	*	value			|	type		|	description of value
	*	----------------|---------------|-------------------------------
	*	0				|	int			|	成功
	*	其他			|	int			|	失败
	*	@note 
	*/
	ZKINTERFACE int APICALL palmSensorSetParameter(void *handle, int paramCode, int paramValue);

	/**
	*	@brief	获取设备参数扩展
	*	@param	:
	*	参数说明如下表
	*	name			|	type		|	param direction		|	description of param
	*	----------------|---------------|-----------------------|------------------------
	*	handle			|	void*		|	[in]				|	设备操作实例句柄
	*	paramCode		|	int			|	[in]				|	参数类型
	*	paramValue		|	char*		|	[out]				|	缓冲区指针地址
	*	paramLen		|	int*		|	[in/out]			|	in:分配缓冲区大小;out:实际参数占用大小
	*	@return
	*	返回值说明如下：
	*	value			|	type		|	description of value
	*	----------------|---------------|-------------------------------
	*	0				|	int			|	成功
	*	其他			|	int			|	失败
	*	@note 
	*/
	ZKINTERFACE int APICALL palmSensorGetParameterEx(void *handle, int paramCode, char *paramValue, int *paramLen);

	/**
	*	@brief	设置设备参数扩展
	*	@param	:
	*	参数说明如下表
	*	name			|	type		|	param direction		|	description of param
	*	----------------|---------------|-----------------------|------------------------
	*	handle			|	void*		|	[in]				|	设备操作实例句柄
	*	paramCode		|	int			|	[in]				|	参数类型
	*	paramValue		|	char*		|	[in]				|	缓冲区指针地址
	*	paramLen		|	int			|	[in]				|	分配缓冲区大小
	*	@return
	*	返回值说明如下：
	*	value			|	type		|	description of value
	*	----------------|---------------|-------------------------------
	*	0				|	int			|	成功
	*	其他			|	int			|	失败
	*	@note 
	*/
	ZKINTERFACE int APICALL palmSensorSetParameterEx(void *handle, int paramCode, char *paramValue, int paramLen);

#ifdef __cplusplus
};
#endif
#else


typedef int (* P_palmSensorInit)();
typedef int (* P_palmSensorFree)();
typedef int (* P_palmSensorGetCount)();
typedef int (* P_palmSensorGetVersion)(char *version, int len);
typedef void* (* P_palmSensorOpen)(int index);
typedef int (* P_palmSensorClose)(void *handle);
typedef int (* P_palmSensorCapture)(void *handle, unsigned char *imageBuffer, int imageBufferSize);
typedef int (* P_palmSensorGetParameter)(void *handle, int paramCode);
typedef int (* P_palmSensorSetParameter)(void *handle, int paramCode, int paramValue);
typedef int (* P_palmSensorGetParameterEx)(void *handle, int paramCode, char *paramValue, int *paramLen);
typedef int (* P_palmSensorSetParameterEx)(void *handle, int paramCode, char *paramValue, int paramLen);
#endif


#endif
