#include "zksensor.h"
#include "../../zkglobaldef.h"
#include"include/ZKPalmErrno.h"
#include <QLibrary>



#ifdef _WIN32
#define ZKPalmAPI_LIBNAME "ZKPalmCap.dll"
#endif
#ifdef __linux
#define ZKPalmAPI_LIBNAME "libzkpalmcap.so"
#endif


ZKSensor::ZKSensor(QObject *parent) : QObject(parent)
{
    m_hDevice = NULL;
    m_nDeviceCount = 0;
    m_pcImgRaw = new unsigned char[MAX_IMG_SIZE];
    memset(m_pcImgRaw,0x0,MAX_IMG_SIZE);
    m_pcVersion= new char[SIZE_64];
    memset(m_pcVersion,0x0,SIZE_64);
}
ZKSensor::~ZKSensor()
{
    m_hDevice = NULL;
    m_nDeviceCount = 0;
    if(m_pcImgRaw)
    {
        delete[] m_pcImgRaw;
        m_pcImgRaw = NULL;
    }
    if(m_pcVersion)
    {
        delete[] m_pcVersion;
        m_pcVersion = NULL;
    }
}

bool ZKSensor::loadLibrary()
{
    QString msg;
    QLibrary libzkbiometric(ZKPalmAPI_LIBNAME);
    if(!libzkbiometric.load())
    {
        msg=QString::fromLocal8Bit("%1 load failed\n").arg(ZKPalmAPI_LIBNAME);
        LOGDEBUG << msg;
        //signalSendMsg(msg);
        return false;
    }
    palmSensorInit = (P_palmSensorInit)libzkbiometric.resolve("palmSensorInit");      LOGDEBUG << "";
    palmSensorFree = (P_palmSensorFree)libzkbiometric.resolve("palmSensorFree");  LOGDEBUG << "";
    palmSensorGetCount = (P_palmSensorGetCount)libzkbiometric.resolve("palmSensorGetCount");  LOGDEBUG << "";
    palmSensorGetVersion = (P_palmSensorGetVersion)libzkbiometric.resolve("palmSensorGetVersion");  LOGDEBUG << "";
    palmSensorOpen = (P_palmSensorOpen)libzkbiometric.resolve("palmSensorOpen");  LOGDEBUG << "";
    palmSensorClose = (P_palmSensorClose)libzkbiometric.resolve("palmSensorClose");  LOGDEBUG << "";
    palmSensorCapture = (P_palmSensorCapture)libzkbiometric.resolve("palmSensorCapture");  LOGDEBUG << "";
    palmSensorGetParameter = (P_palmSensorGetParameter)libzkbiometric.resolve("palmSensorGetParameter");  LOGDEBUG << "";
    palmSensorSetParameter = (P_palmSensorSetParameter)libzkbiometric.resolve("palmSensorSetParameter");  LOGDEBUG << "";
    palmSensorGetParameterEx = (P_palmSensorGetParameterEx)libzkbiometric.resolve("palmSensorGetParameterEx");  LOGDEBUG << "";
    palmSensorSetParameterEx = (P_palmSensorSetParameterEx)libzkbiometric.resolve("palmSensorSetParameterEx");  LOGDEBUG << "";

    if( NULL == palmSensorInit ||
            NULL ==   palmSensorFree ||
            NULL == palmSensorGetCount ||
            NULL == palmSensorGetVersion ||
            NULL == palmSensorOpen ||
            NULL == palmSensorClose ||
            NULL == palmSensorCapture ||
            NULL == palmSensorGetParameter ||
            NULL == palmSensorSetParameter ||
            NULL == palmSensorGetParameterEx ||
            NULL == palmSensorSetParameterEx )
    {
        //ui->textEdit->append(tr("函数接口加载失败"));
        return false;
    }
    return true;
}

bool ZKSensor::open()
{
    int nRet = 0;
    QString msg = NULL;

    if(!loadLibrary())
    {
        msg=QString::fromLocal8Bit("sensor library load failed\n");
        signalSendMsg(msg);
        LOGDEBUG << msg;
        return false ;
    }

    if(NULL != m_hDevice)
    {
        msg=QString::fromLocal8Bit("采集器已打开！");
        signalSendMsg(msg);
        LOGDEBUG << msg;
        return true;
    }

    //get sdk version
    memset(m_pcVersion,0x0,SIZE_64);
    nRet = palmSensorGetVersion(m_pcVersion,SIZE_64);
      LOGDEBUG << "";
    if(ZKPALM_ERR_OK != nRet )
    {
        msg=QString::fromLocal8Bit("获取SDK版本号失败,error code:%1\n").arg(nRet);
        signalSendMsg(msg);
        LOGDEBUG << msg;
        return false;
    }
    msg=QString::fromLocal8Bit("采集库SDK版本号 Version:%1").arg(m_pcVersion);
    signalSendMsg(msg);
    LOGDEBUG << msg;

    // sensor device list init
    nRet = palmSensorInit();
    if(ZKPALM_ERR_OK != nRet )
    {
        msg=QString::fromLocal8Bit("设备初始化失败,error code:%1\n").arg(nRet);
        signalSendMsg(msg);
        LOGDEBUG << msg;
        return false;
    }
    msg=QString::fromLocal8Bit("设备初始化成功");
    signalSendMsg(msg);
    LOGDEBUG << msg;

    //get device count
    m_nDeviceCount = palmSensorGetCount();
    if (m_nDeviceCount <= 0)
    {
        msg=QString::fromLocal8Bit("找不到采集设备,error code:%1").arg(m_nDeviceCount);
        signalSendMsg(msg);
        LOGDEBUG << msg;
        palmSensorFree();
        return false;
    }
    msg=QString::fromLocal8Bit("找到%1采集设备！").arg(m_nDeviceCount);
    signalSendMsg(msg);
    LOGDEBUG << msg;

    //sensor open
    m_hDevice  = palmSensorOpen(0);
    if(NULL==m_hDevice)
    {
        msg=QString::fromLocal8Bit("采集设备打开失败！");
        signalSendMsg(msg);
        palmSensorFree();
        return false;
    }
    else
    {
        msg=QString::fromLocal8Bit("采集设备打开成功！");
        signalSendMsg(msg);
        LOGDEBUG << msg;
    }

    m_nImgWidth = palmSensorGetParameter(m_hDevice,1);
    m_nImgHeight = palmSensorGetParameter(m_hDevice,2);
    msg=QString::fromLocal8Bit("图片宽:%1 高:%2").arg(m_nImgWidth).arg(m_nImgHeight);
    signalSendMsg(msg);
    LOGDEBUG << msg;
    signalStartLoopCapture();
    return true;
}

void ZKSensor::close()
{
    int nRet = 0;
    QString msg = NULL;
    if(NULL == m_hDevice)
    {
        msg=QString::fromLocal8Bit("设备未打开,请先打开设备\n");
    }
    else
    {
        memset(m_pcImgRaw,0x0,MAX_IMG_SIZE);
        memset(m_pcVersion,0x0,SIZE_64);
        mutex.lock();
        nRet = palmSensorClose(m_hDevice);
        m_hDevice = NULL;
        if(ZKPALM_ERR_OK == nRet)
        {
            msg=QString::fromLocal8Bit("设备关闭成功\n");
            palmSensorFree();
        }
        else
        {
            msg=QString::fromLocal8Bit("设备关闭失败,error code:%1\n").arg(nRet);
        }
        mutex.unlock();
    }
    signalSendMsg(msg);
    LOGDEBUG << msg;
}

void ZKSensor::getParam()
{
    int nRet = 0;
    QString msg = NULL;
    int nParamCode = 0;
    char szParamValue[128] = {0x0};
    int nParamValueLen = 128;

    mutex.lock();
    if(NULL == m_hDevice)
    {
        msg=QString::fromLocal8Bit("设备未打开,请先打开设备\n");
        LOGDEBUG << msg;
        signalSendMsg(msg);
    }
    else
    {
        /*
             * get image width
            */
        nParamCode = 1;
        nRet = palmSensorGetParameter(m_hDevice, 1);
        if(nRet>0)
        {
            msg=QString::fromLocal8Bit("图片宽:%1").arg(nRet);
            LOGDEBUG << msg;
            signalSendMsg(msg);
        }

        /*
         * get image height
         */
        nParamCode = 2;
        nRet = palmSensorGetParameter(m_hDevice, 2);
        if(nRet>0)
        {
            msg=QString::fromLocal8Bit("图片高:%1").arg(nRet);
            LOGDEBUG << msg;
            signalSendMsg(msg);
        }

        /*
         * get fw version
         */
        nParamCode = 10010;
        nRet = palmSensorGetParameter(m_hDevice,nParamCode);
        if(nRet > 0)
        {
            // 高字节为主版本号
            int major = (nRet & 0xFF00) >> 8;
            // 低字节为次版本号
            int minjor = nRet & 0xFF;
            msg=QString::fromLocal8Bit("设备固件版本号:V%1.%2").arg(major).arg(minjor);
            LOGDEBUG << msg;
            signalSendMsg(msg);
        }

        /*
          * get device sn
          */
        nParamCode = 1103;
        nRet = palmSensorGetParameterEx(m_hDevice,nParamCode,szParamValue, &nParamValueLen);
        if(ZKPALM_ERR_OK ==  nRet)
        {
            msg=QString::fromLocal8Bit("设备物理序列号:%1").arg(szParamValue);
            LOGDEBUG << msg;
            signalSendMsg(msg);
        }
    }
    mutex.unlock();
    return;
}

void ZKSensor::setParam(int paramCode,int paramValue)
{
    //LOGDEBUG;
    int nRet = 0;
    QString msg = NULL;
    if(NULL == m_hDevice)
    {
        msg=QString::fromLocal8Bit("设备未打开,请先打开设备\n");
        LOGDEBUG << msg;
        signalSendMsg(msg);
    }
    else
    {
        if(2010==paramCode && 0x03==paramValue)
        {
            mutex.lock();
            nRet = palmSensorSetParameter(m_hDevice, paramCode,paramValue);
            mutex.unlock();
            if(0 == nRet)
            {
                msg=QString::fromLocal8Bit(":设备提取模板成功 通知设备成功");
                LOGDEBUG << msg;
            }
            else
            {
                msg=QString::fromLocal8Bit(":设备提取模板成功 通知设备失败,error code:%1").arg(nRet);
                LOGDEBUG << msg;
                signalSendMsg(msg);
            }
        }
        else
        {
            mutex.lock();
            nRet = palmSensorSetParameter(m_hDevice, paramCode,paramValue);
            mutex.unlock();
            if(0 == nRet)
            {
                msg=QString::fromLocal8Bit(":设置参数成功");
                LOGDEBUG << msg;
            }
            else
            {
                msg=QString::fromLocal8Bit(":设置参数失败,error code:%1").arg(nRet);
                LOGDEBUG << msg;
            }
            signalSendMsg(msg);
        }
    }
    mutex.unlock();
    return;
}

void ZKSensor::capture()
{
    //LOGDEBUG;
    int nRet = 0;
    QString msg = NULL;
    memset(m_pcImgRaw,0x0,MAX_IMG_SIZE);
    if(mutex.tryLock())
    {
        if(NULL == m_hDevice)
        {
            mutex.unlock();
        }
        else
        {
            nRet = palmSensorCapture(m_hDevice, m_pcImgRaw, MAX_IMG_SIZE);
            mutex.unlock();
            if(0 <nRet)
            {
                msg=QString("palmSensorCapture nRet=%1").arg(nRet);
                LOGDEBUG <<msg;
                QImage image(m_pcImgRaw, m_nImgWidth, m_nImgHeight, QImage::Format_Indexed8);
                QVector<QRgb> grayTable;
                for(int i = 0; i < 256; i++)
                    grayTable.push_back(qRgb(i,i,i));
                image.setColorTable(grayTable);

                if(!image.isNull())
                {
                    signalSendImage(image);
                    signalSendImageEx(m_pcImgRaw,m_nImgWidth,m_nImgHeight);
                }
            }
            else
            {
                //msg=QString("palmSensorCapture nRet=%1").arg(nRet);
                //LOGDEBUG <<msg;
            }
        }
    }
}

void ZKSensor::slotStartLoopCapture()
{
    LOGDEBUG;
    while(NULL  != m_hDevice)
    {
        capture();
    }
    LOGDEBUG;
}
